#!/usr/bin/python2.7

# Import needed libraries
import RPi.GPIO as GPIO
import os
import sys
import time
import mysql
import mysql.connector
import wiringpi2 as wiringpi
from threading import Thread

# Global variables

##################################
# CONFIG START
##################################
# PIN assignment
# PIN = 1 -- 3.3V
# PIN = 2 -- 5V
# PIN = 3 -- used by I2C Bus
# PIN = 4 -- 5V
# PIN = 5 -- used by I2C Bus
# PIN = 6 -- GND
# PIN = 7 -- used by kernel module, 1-Wire Bus, temperature sensors
PIN_SERVO_OUTER_DATA = 8
# PIN = 9 -- GND
PIN_SERVO_INNER_DATA = 10
PIN_SOLENOID_OUTER = 11
PIN_PWM_BUZZER = 12
PIN_SOLENOID_INNER = 13
# PIN_14 = 14 -- GND
PIN_DOOR_INNER = 15
PIN_SERVO_INNER_POWER = 16
# PIN_17 = 17 -- 3.3V
PIN_SERVO_OUTER_POWER = 18
PIN_DOOR_OUTER = 19
# PIN_20 = 20 -- GND
PIN_LEDS_INNER = 21
PIN_PELTIER_TOP = 22
PIN_LEDS_OUTER = 23
PIN_PELTIER_BOTTOM_RIGHT = 24
# PIN_25 = 25 -- GND
PIN_PELTIER_BOTTOM_LEFT = 26

# Database connection
DBHOST = "localhost"
DBUSER = "root"
DBPASSWD = "m3SS2014coci"
DBNAME = "isolocker"

# Database value management
DB_GUEST_LOGIN_CLEANER_THRESHOLD = 600
DB_GUEST_LOGIN_CLEANER_INTERVAL = 120

# Ids
MAIN_SHELF_ID = 1
MAIN_IGNORE_USER_ID = 1

# Performance setting
MAIN_DELAY_TIMER = 0.25
TEMP_DELAY_TIMER = 0.5
BUZZER_DELAY_TIMER = 0.5
RGB_DELAY_TIMER = 0.25

# Cooldown settings
MAIN_SOLENOID_COOLDOWN_SECONDS = 10
MAIN_PELTIER_COOLDOWN_SECONDS = 30
MAIN_PELTIER_ADDITIONAL_COOLING_TEMPERATURE = 1.0

# I2C addresses
I2C_ADDR_LIGHT_LEFT = 0x29
I2C_ADDR_LIGHT_RIGHT = 0x39
I2C_ADDR_RGB_OUTER = 0x09
I2C_ADDR_RGB_INNER = 0x0a

# 1-Wire filepaths and temperature invalid threshold
FILEPATH_1WIRE_UPPER = "/sys/bus/w1/devices/28-00000529500d/w1_slave"
FILEPATH_1WIRE_LOWER = "/sys/bus/w1/devices/28-00000529973d/w1_slave"
TEMP_INVALID_THRESHOLD = 5

# Light threshold, when both values are below these values, outer LEDs will turn on
LIGHT_INFRA_THRESHOLD = 5
LIGHT_VISIBLE_THRESHOLD = 12
LIGHT_INVALID_THRESHOLD = 20

# Servo values, depend on your setup, determined by testing carefully
SERVO_INNER_OPEN_POSITION = 7.8
SERVO_INNER_CLOSED_POSITION = 3.5
SERVO_OUTER_OPEN_POSITION = 7.7
SERVO_OUTER_CLOSED_POSITION = 11.3

# RGB settings
RGB_BLINK_COOLING_RANGE_AROUND_TARGET_TEMP = 1.0
RGB_BLINK_COLOR_STR = "blue"
RGB_BLINK_INTERVAL_SECONDS = 60
RGB_BLINK_TIMES = 3
RGB_BLINK_TIMER_SECONDS = 3
RGB_RESEND_TIMER_SECONDS = 3

# BUZZER settings
BUZZER_TIMER = 0.5
BUZZER_DOOR_OPEN_TIMER_SECONDS = 30

##################################
# CONFIG END
##################################

# Get root rights for execution, if we are not already root
euid = os.geteuid()
if euid != 0:
    print "Not running as root, using sudo to gain root rights"
    args = ["sudo", sys.executable] + sys.argv + [os.environ]
    os.execlpe("sudo", *args)
else:
    print "Running as root now."

# Initial PIN states
GPIO.setmode(GPIO.BOARD)
GPIO.setup(PIN_DOOR_OUTER, GPIO.IN, pull_up_down = GPIO.PUD_DOWN)              # Door button - Outer door
GPIO.setup(PIN_DOOR_INNER, GPIO.IN, pull_up_down = GPIO.PUD_DOWN)              # Door button - Inner door
GPIO.setup(PIN_PELTIER_TOP, GPIO.OUT)                                          # Peltier top
GPIO.setup(PIN_PELTIER_BOTTOM_RIGHT, GPIO.OUT)                                 # Peltier bottom right
GPIO.setup(PIN_PELTIER_BOTTOM_LEFT, GPIO.OUT)                                  # Peltier bottom left
GPIO.setup(PIN_LEDS_OUTER, GPIO.OUT)                                           # LEDs outer
GPIO.setup(PIN_LEDS_INNER, GPIO.OUT)                                           # LEDs inner
GPIO.setup(PIN_SERVO_INNER_POWER, GPIO.OUT)                                    # Servo inner power
GPIO.setup(PIN_SERVO_INNER_DATA, GPIO.OUT)                                     # Servo inner data
GPIO.setup(PIN_SERVO_OUTER_POWER, GPIO.OUT)                                    # Servo outer power
GPIO.setup(PIN_SERVO_OUTER_DATA, GPIO.OUT)                                     # Servo outer data
GPIO.setup(PIN_SOLENOID_INNER, GPIO.OUT)                                       # Solenoid inner
GPIO.setup(PIN_SOLENOID_OUTER, GPIO.OUT)                                       # Solenoid outer


# WiringPi setup
wiringpi.wiringPiSetupPhys()
i2cDevLightLeft = wiringpi.wiringPiI2CSetup(I2C_ADDR_LIGHT_LEFT)
i2cDevLightRight = wiringpi.wiringPiI2CSetup(I2C_ADDR_LIGHT_RIGHT)
i2cDevRgbOuter = wiringpi.wiringPiI2CSetup(I2C_ADDR_RGB_OUTER)
i2cDevRgbInner = wiringpi.wiringPiI2CSetup(I2C_ADDR_RGB_INNER)

# Sensor variables from other threads
lightVisibleLeft = 0
lightInfraLeft = 0
lightVisibleRight = 0
lightInfraRight = 0

tempUpper = 0
tempLower = 0

# Actor variables
buzzerActive = 0
doorInnerOpeningInProgress = 0
doorOuterOpeningInProgress = 0
doorInnerClosingInProgress = 0
doorOuterClosingInProgress = 0
rgbInnerColorInProgress = 0
rgbInnerBlinkInProgress = 0
rgbOuterColorInProgress = 0
rgbOuterBlinkInProgress = 0
rgbSetColorString = ""

# Threading
lightLeftThreadRunOnce = 0
lightRightThreadRunOnce = 0
tempUpperThreadRunOnce = 0
tempLowerThreadRunOnce = 0
rgbInnerInitThreadRunOnce = 0
rgbOuterInitThreadRunOnce = 0

##################################
# DOOR CONTROL THREAD START
##################################

def doorControlThread(inner, open):

    global PIN_SERVO_INNER_POWER, PIN_SERVO_INNER_DATA, SERVO_INNER_OPEN_POSITION, SERVO_INNER_CLOSED_POSITION, \
    PIN_SERVO_OUTER_POWER, PIN_SERVO_OUTER_DATA, SERVO_OUTER_OPEN_POSITION, SERVO_OUTER_CLOSED_POSITION, \
    PIN_SOLENOID_INNER, PIN_SOLENOID_OUTER, \
    doorInnerOpeningInProgress, doorOuterOpeningInProgress, doorInnerClosingInProgress, doorOuterClosingInProgress

    pinData = 0
    pinPower = 0
    pinSolenoid = 0
    targetValue = 0.0

    if (inner):
        pinData = PIN_SERVO_INNER_DATA
        pinPower = PIN_SERVO_INNER_POWER
        pinSolenoid = PIN_SOLENOID_INNER

        if (open):
            targetValue = SERVO_INNER_OPEN_POSITION
        else:
            targetValue = SERVO_INNER_CLOSED_POSITION

    else:
        pinData = PIN_SERVO_OUTER_DATA
        pinPower = PIN_SERVO_OUTER_POWER
        pinSolenoid = PIN_SOLENOID_OUTER

        if (open):
            targetValue = SERVO_OUTER_OPEN_POSITION
        else:
            targetValue = SERVO_OUTER_CLOSED_POSITION

    GPIO.output(pinPower, GPIO.HIGH)
    GPIO.setup(pinData, GPIO.OUT)
    servo = GPIO.PWM(pinData, 50)
    servo.start(targetValue)
    time.sleep(0.1)

    if (open):
        GPIO.output(pinSolenoid, GPIO.HIGH)

    time.sleep(0.6)

    if (open):
        GPIO.output(pinSolenoid, GPIO.LOW)

    servo.stop()
    GPIO.setup(pinData, GPIO.IN, pull_up_down = GPIO.PUD_DOWN)
    GPIO.output(pinPower, GPIO.LOW)

    if (inner and open):
        doorInnerOpeningInProgress = 0

    if ((not inner) and open):
        doorOuterOpeningInProgress = 0

    if (inner and (not open)):
        doorInnerClosingInProgress = 0

    if ((not inner) and (not open)):
        doorOuterClosingInProgress = 0

##################################
# DOOR CONTROL THREAD END
##################################

##################################
# MAIN THREAD START
##################################

# Infinite loop doing everything
def mainThread():

    global PIN_DOOR_INNER, PIN_DOOR_OUTER, PIN_LEDS_INNER, PIN_LEDS_OUTER, PIN_PELTIER_TOP, PIN_PELTIER_BOTTOM_RIGHT, PIN_PELTIER_BOTTOM_LEFT, \
    DBHOST, DBUSER, DBPASSWD, DBNAME, \
    DB_GUEST_LOGIN_CLEANER_THRESHOLD, DB_GUEST_LOGIN_CLEANER_INTERVAL, \
    MAIN_SHELF_ID, MAIN_IGNORE_USER_ID, \
    MAIN_DELAY_TIMER, \
    MAIN_SOLENOID_COOLDOWN_SECONDS, MAIN_PELTIER_COOLDOWN_SECONDS, MAIN_PELTIER_ADDITIONAL_COOLING_TEMPERATURE, \
    RGB_BLINK_COOLING_RANGE_AROUND_TARGET_TEMP, RGB_BLINK_INTERVAL_SECONDS, RGB_RESEND_TIMER_SECONDS, \
    BUZZER_DOOR_OPEN_TIMER_SECONDS, \
    lightVisibleLeft, lightInfraLeft, lightVisibleRight, lightInfraRight, lightLeftThreadRunOnce, lightRightThreadRunOnce, \
    tempUpper, tempLower, tempUpperThreadRunOnce, tempLowerThreadRunOnce, \
    rgbInnerColorInProgress, rgbInnerBlinkInProgress, rgbOuterColorInProgress, rgbOuterBlinkInProgress, rgbInnerInitThreadRunOnce, rgbOuterInitThreadRunOnce, rgbSetColorString, \
    doorInnerOpeningInProgress, doorOuterOpeningInProgress, doorInnerClosingInProgress, doorOuterClosingInProgress, \
    buzzerActive

    # Establish database connection
    while True:
        try:
            connection = mysql.connector.connect(host = DBHOST, user = DBUSER, passwd = DBPASSWD, db = DBNAME)
            break
        except:
            print "MySQL connection could not be established, Retry in 5s..."
            time.sleep(5)

    # Database cursor
    cursor = connection.cursor()

    # Local sensor / actor variables
    lightVisibleAverage = 0
    lightInfraAverage = 0
    tempAverage = 0
    doorInnerOpenNow = 0
    doorOuterOpenNow = 0
    coolingActive = 0

    # Local Timers
    timestampScriptStart = time.time()
    timestampDbChecked = time.time()
    timestampCurrent = time.time()
    timestampPeltierCooldown = 0
    timestampDoorInnerOpened = time.time()
    timestampDoorOuterOpened = time.time()
    timestampRgbBlinked = 0
    timestampRgbResend = 0

    # Default variables for target states from website - SQL
    user_id_tmp = MAIN_IGNORE_USER_ID
    target_temperature_tmp = 0
    cooling_active_tmp = 0
    target_color_tmp = ""
    target_door_state_inner_tmp = 0
    target_door_state_outer_tmp = 0

    while True:

        ##################################
        # Check other threads run once
        ##################################
        if ((not lightLeftThreadRunOnce) or (not lightRightThreadRunOnce) or (not tempUpperThreadRunOnce) or (not tempLowerThreadRunOnce) or (not rgbInnerInitThreadRunOnce) or (not rgbOuterInitThreadRunOnce)):
            time.sleep(1)
            continue

        ##################################
        # Set current timestamp
        ##################################
        timestampCurrent = time.time()

        ##################################
        # READ VALUES START
        ##################################

        # Variables for detecting door button pressed
        doorInnerButtonPressed = 0
        doorOuterButtonPressed = 0

        # Door button, inner door state
        if (GPIO.input(PIN_DOOR_INNER)):
            if (doorInnerOpenNow):
                doorInnerButtonPressed = 1

            doorInnerOpenNow = 0
        else:
            doorInnerOpenNow = 1

        # Door button, outer door state
        if (GPIO.input(PIN_DOOR_OUTER)):
            if (doorOuterOpenNow):
                doorOuterButtonPressed = 1

            doorOuterOpenNow = 0
        else:
            doorOuterOpenNow = 1

        # Light threshold, take bigger value in case of threshold error
        if (abs(lightVisibleRight - lightVisibleLeft) > LIGHT_INVALID_THRESHOLD):
            if (lightVisibleRight > lightVisibleLeft):
                lightVisibleLeft = lightVisibleRight
            else:
                lightVisibleRight = lightVisibleLeft

        if (abs(lightInfraRight - lightInfraLeft) > LIGHT_INVALID_THRESHOLD):
            if (lightInfraRight > lightInfraLeft):
                lightInfraLeft = lightInfraRight
            else:
                lightInfraRight = lightInfraLeft

        # Calculate average light values
        lightVisibleAverage = (lightVisibleRight + lightVisibleLeft) / 2
        lightInfraAverage = (lightInfraRight + lightInfraLeft) / 2

        # Temperature threshold, take bigger value in case of threshold error
        if (abs(tempLower - tempUpper) > TEMP_INVALID_THRESHOLD):
            if (tempUpper > tempLower):
                tempLower = tempUpper
            else:
                tempUpper = tempLower

        # Calculate average temperature
        tempAverage = (tempUpper + tempLower) / 2

        # Read user values from database
        cursor.execute("SELECT user_id, target_temperature, cooling_active, target_color, target_door_state_inner, target_door_state_outer FROM shelf WHERE shelf_id = " + str(MAIN_SHELF_ID) + " LIMIT 1")
        for (user_id, target_temperature, cooling_active, target_color, target_door_state_inner, target_door_state_outer) in cursor:
            user_id_tmp = user_id
            target_temperature_tmp = float(target_temperature)
            cooling_active_tmp = cooling_active
            target_color_tmp = target_color
            target_door_state_inner_tmp = target_door_state_inner
            target_door_state_outer_tmp = target_door_state_outer

        # When button was pressed, then door state is not up to date and needs to be set to 0 in next db update
        if (doorInnerButtonPressed):
            target_door_state_inner_tmp = 0

        if (doorOuterButtonPressed):
            target_door_state_outer_tmp = 0

        # When belongs to nobody, no color and no cooling
        if (user_id_tmp == MAIN_IGNORE_USER_ID):
            cooling_active_tmp = 0
            target_color_tmp = "black"

        ##################################
        # READ VALUES END
        ##################################

        ##################################
        # ACTIONS START
        ##################################

        # LEDs Outer: Turn light outer on if it is dark
        if ((lightVisibleAverage < LIGHT_VISIBLE_THRESHOLD) and (lightInfraAverage < LIGHT_INFRA_THRESHOLD)):
            GPIO.output(PIN_LEDS_OUTER, GPIO.HIGH)
        else:
            GPIO.output(PIN_LEDS_OUTER, GPIO.LOW)

        # Cooling with peltier elements active, if temperatures are not reached, cooling active and cooldown allows it
        if (((target_temperature_tmp - MAIN_PELTIER_ADDITIONAL_COOLING_TEMPERATURE) < tempAverage) and (cooling_active_tmp == 1)):
            if ((timestampCurrent - timestampPeltierCooldown) >= MAIN_PELTIER_COOLDOWN_SECONDS):
                GPIO.output(PIN_PELTIER_TOP, GPIO.HIGH)
                GPIO.output(PIN_PELTIER_BOTTOM_RIGHT, GPIO.HIGH)
                GPIO.output(PIN_PELTIER_BOTTOM_LEFT, GPIO.HIGH)
                coolingActive = 1
        else:
            GPIO.output(PIN_PELTIER_TOP, GPIO.LOW)
            GPIO.output(PIN_PELTIER_BOTTOM_RIGHT, GPIO.LOW)
            GPIO.output(PIN_PELTIER_BOTTOM_LEFT, GPIO.LOW)

            if (coolingActive):
                timestampPeltierCooldown = timestampCurrent
                coolingActive = 0

        # Only do changes if no thread is in progress
        if ((not rgbInnerColorInProgress) and (not rgbInnerBlinkInProgress) and (not rgbOuterColorInProgress) and (not rgbOuterBlinkInProgress)):

            # RGB control, color change
            if ((target_color_tmp != rgbSetColorString) or ((timestampCurrent - timestampRgbResend) > RGB_RESEND_TIMER_SECONDS)):
                rgbSetColorString = target_color_tmp
                timestampRgbResend = timestampCurrent
                rgbInnerColorInProgress = 1
                rgbOuterColorInProgress = 1

            # RGB control, blink
            if ((target_color_tmp == rgbSetColorString) and ((timestampCurrent - timestampRgbBlinked) > RGB_BLINK_INTERVAL_SECONDS) and ((target_temperature_tmp - RGB_BLINK_COOLING_RANGE_AROUND_TARGET_TEMP) <= tempAverage) and (tempAverage <= (target_temperature_tmp + RGB_BLINK_COOLING_RANGE_AROUND_TARGET_TEMP)) and (cooling_active_tmp == 1)):
                timestampRgbBlinked = timestampCurrent
                rgbInnerBlinkInProgress = 1
                rgbOuterBlinkInProgress = 1

        # Door handling
        # Open inner door
        if (target_door_state_inner_tmp and (not doorInnerOpenNow) and (not doorInnerOpeningInProgress) and (not doorInnerClosingInProgress) \
        and (not doorOuterOpeningInProgress) and ((timestampCurrent - timestampDoorInnerOpened) > MAIN_SOLENOID_COOLDOWN_SECONDS)):
            doorInnerOpenNow = 1
            doorInnerOpeningInProgress = 1
            timestampDoorInnerOpened = timestampCurrent
            doorThread = Thread(target=doorControlThread, args=(1, 1,))
            doorThread.start()

        # Open outer door
        if (target_door_state_outer_tmp and (not doorOuterOpenNow) and (not doorOuterOpeningInProgress) and (not doorOuterClosingInProgress) \
        and (not doorInnerOpeningInProgress) and ((timestampCurrent - timestampDoorOuterOpened) > MAIN_SOLENOID_COOLDOWN_SECONDS)):
            doorOuterOpenNow = 1
            doorOuterOpeningInProgress = 1
            timestampDoorOuterOpened = timestampCurrent
            doorThread = Thread(target=doorControlThread, args=(0, 1,))
            doorThread.start()

        # Close inner door
        if (doorInnerButtonPressed and (not doorInnerOpenNow) and (not doorInnerOpeningInProgress) and (not doorInnerClosingInProgress)):
            target_door_state_inner_tmp = 0
            doorInnerOpenNow = 0
            doorInnerClosingInProgress = 1
            doorThread = Thread(target=doorControlThread, args=(1, 0,))
            doorThread.start()

        # Close outer door
        if (doorOuterButtonPressed and (not doorOuterOpenNow) and (not doorOuterOpeningInProgress) and (not doorOuterClosingInProgress)):
            target_door_state_outer_tmp = 0
            doorOuterOpenNow = 0
            doorOuterClosingInProgress = 1
            doorThread = Thread(target=doorControlThread, args=(0, 0,))
            doorThread.start()

        # Buzzer active when door open for too long
        if ((doorInnerOpenNow and ((timestampCurrent - timestampDoorInnerOpened) > BUZZER_DOOR_OPEN_TIMER_SECONDS)) \
        or (doorOuterOpenNow and ((timestampCurrent - timestampDoorOuterOpened) > BUZZER_DOOR_OPEN_TIMER_SECONDS))):
            buzzerActive = 1
        else:
            buzzerActive = 0

        # LEDs Inner: Turn light inner on if a door is opened
        if (doorOuterOpenNow == 1 or doorInnerOpenNow == 1):
            GPIO.output(PIN_LEDS_INNER, GPIO.HIGH)
        else:
            GPIO.output(PIN_LEDS_INNER, GPIO.LOW)

        ##################################
        # ACTIONS END
        ##################################

        ##################################
        # DB VALUES UPDATE START
        ##################################

        if ((timestampCurrent - timestampDbChecked) > DB_GUEST_LOGIN_CLEANER_INTERVAL):
            timestampDbChecked = timestampCurrent
            cursor.execute("DELETE FROM guest WHERE shelf_id = " + str(MAIN_SHELF_ID) + " AND (UNIX_TIMESTAMP() - session_timestamp) > " + str(DB_GUEST_LOGIN_CLEANER_THRESHOLD))
            connection.commit()            

        tempStr = str(tempAverage)
        tempStrSplit = tempStr.split(".")
        tempAverageStr = tempStrSplit[0] + "." + tempStrSplit[1][0]
        cursor.execute("UPDATE shelf SET current_temperature = " + tempAverageStr + ", target_door_state_inner = " + str(target_door_state_inner_tmp) + ", current_door_state_inner = " + str(doorInnerOpenNow) + ", target_door_state_outer = " + str(target_door_state_outer_tmp) + ", current_door_state_outer = " + str(doorOuterOpenNow) + " WHERE shelf_id = " + str(MAIN_SHELF_ID) + " LIMIT 1")
        connection.commit()

        ##################################
        # DB VALUES UPDATE END
        ##################################

        # Delay for next loop
        time.sleep(MAIN_DELAY_TIMER)

##################################
# MAIN THREAD END
##################################

##################################
# READ LIGHT THREAD START
##################################

def lightReadThread(right):

    global i2cDevLightRight, lightVisibleRight, lightInfraRight, lightRightThreadRunOnce, \
    i2cDevLightLeft, lightVisibleLeft, lightInfraLeft, lightLeftThreadRunOnce

    dev = 0

    if (right):
        dev = i2cDevLightRight
    else:
        dev = i2cDevLightLeft

    while True:
        # I2C Light sensor
        wiringpi.wiringPiI2CWriteReg8(dev, 0x80, 0x03)
        time.sleep(0.5)
        data1 = wiringpi.wiringPiI2CReadReg8(dev, 0x80 | 0x0C)
        data2 = wiringpi.wiringPiI2CReadReg8(dev, 0x80 | 0x0D)
        data3 = wiringpi.wiringPiI2CReadReg8(dev, 0x80 | 0x0E)
        data4 = wiringpi.wiringPiI2CReadReg8(dev, 0x80 | 0x0F)
        wiringpi.wiringPiI2CWriteReg8(dev, 0x80, 0x00)

        if (right):
            lightVisibleRight = 256 * data2 + data1
            lightInfraRight = 256 * data4 + data3
            lightRightThreadRunOnce = 1
        else:
            lightVisibleLeft = 256 * data2 + data1
            lightInfraLeft = 256 * data4 + data3
            lightLeftThreadRunOnce = 1

##################################
# READ LIGHT THREAD END
##################################

##################################
# READ TEMP THREAD START
##################################

def tempReadThread(upper):

    global FILEPATH_1WIRE_UPPER, FILEPATH_1WIRE_LOWER, \
    TEMP_DELAY_TIMER, \
    tempUpper, tempLower, \
    tempUpperThreadRunOnce, tempLowerThreadRunOnce

    filePath = ""

    if (upper):
        filePath = FILEPATH_1WIRE_UPPER
    else:
        filePath = FILEPATH_1WIRE_LOWER

    while True:
        # Read temperature file, 1-wire bus
        try:
            strFromFile = ""

            with open(filePath, "r") as tmpFile:
                strFromFile = tmpFile.read().replace("\n", "")

            if (upper):
                tempUpper = strFromFile.split()[-1]
                tempUpper = tempUpper.replace("t=", "")
                tempUpper = float(tempUpper)
                tempUpper = tempUpper / 1000
                tempUpperThreadRunOnce = 1
            else:
                tempLower = strFromFile.split()[-1]
                tempLower = tempLower.replace("t=", "")
                tempLower = float(tempLower)
                tempLower = tempLower / 1000
                tempLowerThreadRunOnce = 1
        except:
            if (upper):
                tempUpper = 0
                tempUpperThreadRunOnce = 1
            else:
                tempLower = 0
                tempLowerThreadRunOnce = 1

        time.sleep(TEMP_DELAY_TIMER)

##################################
# READ TEMP THREAD END
##################################

##################################
# RGB CONTROL THREAD START
##################################

def rgbControlThread(inner):

    global RGB_BLINK_COLOR_STR, RGB_BLINK_TIMES, RGB_BLINK_TIMER_SECONDS, RGB_DELAY_TIMER, \
    rgbInnerInitThreadRunOnce, rgbOuterInitThreadRunOnce, \
    rgbInnerColorInProgress, rgbInnerBlinkInProgress, rgbOuterColorInProgress, rgbOuterBlinkInProgress, rgbSetColorString, \
    bus, i2cDevRgbInner, i2cDevRgbOuter

    # I2C Bus
    dev = 0

    # Set dev for inner or outer RGB
    if (inner):
        dev = i2cDevRgbInner
    else:
        dev = i2cDevRgbOuter

    # Set startup command do nothing
    wiringpi.wiringPiI2CWrite(dev, ord('B'))
    wiringpi.wiringPiI2CWrite(dev, 0)
    wiringpi.wiringPiI2CWrite(dev, 0)
    wiringpi.wiringPiI2CWrite(dev, 0)
    wiringpi.wiringPiI2CWrite(dev, 0)
    wiringpi.wiringPiI2CWrite(dev, 0)

    # Set fading back to default value
    wiringpi.wiringPiI2CWrite(dev, ord('f'))
    wiringpi.wiringPiI2CWrite(dev, 20)

    # Stop light script
    wiringpi.wiringPiI2CWrite(dev, ord('o'))

    # Direct change to Black
    wiringpi.wiringPiI2CWrite(dev, ord('n'))
    wiringpi.wiringPiI2CWrite(dev, 0x00)
    wiringpi.wiringPiI2CWrite(dev, 0x00)
    wiringpi.wiringPiI2CWrite(dev, 0x00)

    # Default variable values
    rgbColorInProgress = 0
    rgbBlinkInProgress = 0

    if (inner):
        rgbInnerInitThreadRunOnce = 1
    else:
        rgbOuterInitThreadRunOnce = 1

    while True:

        # Evaluate global variables
        if (inner):
            rgbColorInProgress = rgbInnerColorInProgress
            rgbBlinkInProgress = rgbInnerBlinkInProgress
        else:
            rgbColorInProgress = rgbOuterColorInProgress
            rgbBlinkInProgress = rgbOuterBlinkInProgress

        # Different commands
        # Blink, rgbSetColorString contains original color
        if (rgbSetColorString != "" and (not rgbColorInProgress) and rgbBlinkInProgress and RGB_BLINK_TIMES > 0):

            # Fade to Black
            wiringpi.wiringPiI2CWrite(dev, ord('c'))
            wiringpi.wiringPiI2CWrite(dev, 0x00)
            wiringpi.wiringPiI2CWrite(dev, 0x00)
            wiringpi.wiringPiI2CWrite(dev, 0x00)
            time.sleep(RGB_BLINK_TIMER_SECONDS)

            colorValsOne = colorStrToValues(RGB_BLINK_COLOR_STR)

            # Repeat blinking
            for i in range(0, RGB_BLINK_TIMES):
                # Direct change to configured color
                wiringpi.wiringPiI2CWrite(dev, ord('n'))
                wiringpi.wiringPiI2CWrite(dev, colorValsOne[0])
                wiringpi.wiringPiI2CWrite(dev, colorValsOne[1])
                wiringpi.wiringPiI2CWrite(dev, colorValsOne[2])
                time.sleep(RGB_BLINK_TIMER_SECONDS)

                # Direct change to black
                wiringpi.wiringPiI2CWrite(dev, ord('n'))
                wiringpi.wiringPiI2CWrite(dev, 0x00)
                wiringpi.wiringPiI2CWrite(dev, 0x00)
                wiringpi.wiringPiI2CWrite(dev, 0x00)
                time.sleep(RGB_BLINK_TIMER_SECONDS)

            # Fade to previous color
            colorValsTwo = colorStrToValues(rgbSetColorString)
            wiringpi.wiringPiI2CWrite(dev, ord('c'))
            wiringpi.wiringPiI2CWrite(dev, colorValsTwo[0])
            wiringpi.wiringPiI2CWrite(dev, colorValsTwo[1])
            wiringpi.wiringPiI2CWrite(dev, colorValsTwo[2])

            if (inner):
                rgbInnerBlinkInProgress = 0
            else:
                rgbOuterBlinkInProgress = 0

        # Change color, fading
        if (rgbSetColorString != "" and rgbColorInProgress and (not rgbBlinkInProgress)):

            # Fade to color
            colorVals = colorStrToValues(rgbSetColorString)
            wiringpi.wiringPiI2CWrite(dev, ord('c'))
            wiringpi.wiringPiI2CWrite(dev, colorVals[0])
            wiringpi.wiringPiI2CWrite(dev, colorVals[1])
            wiringpi.wiringPiI2CWrite(dev, colorVals[2])

            if (inner):
                rgbInnerColorInProgress = 0
            else:
                rgbOuterColorInProgress = 0

        time.sleep(RGB_DELAY_TIMER)

##################################
# RGB CONTROL THREAD END
##################################

##################################
# BUZZER CONTROL THREAD START
##################################
def buzzerControlThread():

    global PIN_PWM_BUZZER, BUZZER_TIMER, buzzerActive, BUZZER_DELAY_TIMER

    wiringpi.pinMode(PIN_PWM_BUZZER, 2)
    wiringpi.pwmSetMode(0)

    # pwmFrequency in Hz = 19.2e6 Hz / pwmClock / pwmRange
    # valid range for pwmSetClock() seems to go from 2 to 4095
    # valid range for pwmSetRange() is up to 4096
    # duty cycle 50%, pwmWrite half of range
    wiringpi.pwmSetRange(10)
    wiringpi.pwmSetClock(1920)
    wiringpi.pwmWrite(PIN_PWM_BUZZER, 0)

    while True:
        if (buzzerActive):
            wiringpi.pwmWrite(PIN_PWM_BUZZER, 5)
            wiringpi.pwmSetClock(1920)
            time.sleep(BUZZER_TIMER)
            wiringpi.pwmSetClock(3200)
            time.sleep(BUZZER_TIMER)
        else:
            wiringpi.pwmWrite(PIN_PWM_BUZZER, 0)
            time.sleep(BUZZER_DELAY_TIMER)

##################################
# BUZZER CONTROL THREAD END
##################################

##################################
# FUNCTION COLOR => RGB VALUES START
##################################

def colorStrToValues(colorStr):

    resultList = [0x00, 0x00, 0x00]

    if colorStr == "black":
        resultList = [0x00, 0x00, 0x00]
    elif colorStr == "white":
        resultList = [0xFF, 0xFF, 0xFF]
    elif colorStr == "cyan":
        resultList = [0x00, 0xFF, 0xFF]
    elif colorStr == "magenta":
        resultList = [0xFF, 0x00, 0x44]
    elif colorStr == "yellow":
        resultList = [0xFF, 0xFF, 0x00]
    elif colorStr == "orange":
        resultList = [0xFF, 0x44, 0x00]
    elif colorStr == "green":
        resultList = [0x00, 0xFF, 0x00]

    return resultList

##################################
# FUNCTION COLOR => RGB VALUES END
##################################

# Start default threads
mainThread = Thread(target=mainThread, args=())
lightLeftThread = Thread(target=lightReadThread, args=(0,))
lightRightThread = Thread(target=lightReadThread, args=(1,))
tempLowerThread = Thread(target=tempReadThread, args=(0,))
tempUpperThread = Thread(target=tempReadThread, args=(1,))
rgbOuterThread = Thread(target=rgbControlThread, args=(0,))
rgbInnerThread = Thread(target=rgbControlThread, args=(1,))
buzzerControlThread = Thread(target=buzzerControlThread, args=())

mainThread.start()
lightLeftThread.start()
lightRightThread.start()
tempLowerThread.start()
tempUpperThread.start()
rgbOuterThread.start()
rgbInnerThread.start()
buzzerControlThread.start()
